package sbc.mgclus.parser;

import java.io.File;
import java.io.PrintWriter;

import org.apache.commons.cli.*;
import org.apache.log4j.Level;

public class CmdParser {
	private static Options options = null; // Command line options
	
	private static final String graphFileShort = "f";
	private static final String graphFileLong = "file";
	private static final String outputFileShort = "o";
	private static final String outputFileLong = "output";
	private static final String weightsShort = "w";
	private static final String weightsLong = "weights";
	private static final String seedsShort = "s";
	private static final String seedsLong = "seeds";
	private static final String helpShort = "h";
	private static final String helpLong = "help";
	private static final String deltaShort = "d";
	private static final String deltaLong = "delta";
	private static final String verboseShort = "v";
	private static final String verboseLong = "verbose";
	
	private CommandLine cmd = null; // Command Line arguments
	
	static{
		options = new Options();
		options.addOption(graphFileShort, graphFileLong, true, "Graph file location");
		options.addOption(outputFileShort, outputFileLong, true, "Output file locaction (optional)");
		options.addOption(seedsShort, seedsLong, true, "Seed file location (optional)");
		options.addOption(weightsShort, weightsLong, false, "Use edge weights (optional; Boolean)");
		options.addOption(deltaShort, deltaLong, true, "delta; merge cut-off (optional; default=0)");
		options.addOption(verboseShort, verboseLong, true, "level of verbosity (i:info (def), e:error, d:debug)");
		options.addOption(helpShort, helpLong, false, "Display help description");
	}
	
	private File graphFile = null;
	private File outFile = null;
	private Boolean useWeights = false;
	private File seedsFile = null;
	private Double delta = null;
	private Level verbose = Level.INFO;
	

	public CmdParser (String[] args) {
		CommandLineParser parser = new PosixParser();
		HelpFormatter formatter = new HelpFormatter();
		
		try {
			cmd = parser.parse(options, args);
			
		} catch (ParseException e) {
			System.err.println("\nERROR -> error parsing command line arguments\n");
			formatter.printHelp(new PrintWriter(System.err,true), 150, "mgclus -f INFILE [OPTIONS]\n", "", options, 2, 2, "");
			System.err.println("\n");
			System.exit(1);
		}
		
		if(cmd.hasOption("h") || cmd.hasOption("help")) {
			formatter.printHelp(new PrintWriter(System.err,true), 150, "mgclus -f INFILE [OPTIONS]\n", "", options, 2, 2, "");
			System.err.println("\n");
			System.exit(1);
		}

		// Check for mandatory args
		
		if (! cmd.hasOption("f") || ! cmd.hasOption("file")){
			System.err.println("\nERROR -> no network file specified\n");
			formatter.printHelp(new PrintWriter(System.err,true), 150, "mgclus -f INFILE [OPTIONS]\n", "", options, 2, 2, "");
			System.err.println("\n");
			System.exit(1);
		}
		else {
			setGraphFile(new File (cmd.getOptionValue("f")));
		}
		
		// Look for optional args
		
		if(cmd.hasOption("o") || cmd.hasOption("output")) {
			setOutputFile(new File (cmd.getOptionValue("o")));
		}
		
		if(cmd.hasOption("w") || cmd.hasOption("weights")) {
			setUseWeights(true);
		}
		
		if(cmd.hasOption("s") || cmd.hasOption("seeds")) {
			setSeedsFile(new File (cmd.getOptionValue("s")));
		}

		if(cmd.hasOption("d") || cmd.hasOption("delta")) {
			try {
				setDelta(Double.valueOf((cmd.getOptionValue("d")).trim()).doubleValue());
			}
			catch (NumberFormatException nfe) {
				System.err.println("\nERROR -> delta has wrong type: should be a number (default = 0)\n");
				formatter.printHelp(new PrintWriter(System.err,true), 150, "mgclus -f INFILE [OPTIONS]\n", "", options, 2, 2, "");
				System.err.println("\n");
		        //formatter.printHelp( "mgclus", options );
		        System.exit(1);
		    }
		}
		
		if(cmd.hasOption("v") || cmd.hasOption("verbose")) {
			if (cmd.getOptionValue("v").equals("i"))
				setVerbose(Level.INFO);
			else if (cmd.getOptionValue("v").equals("d"))
				setVerbose(Level.DEBUG);
			else if (cmd.getOptionValue("v").equals("e"))
				setVerbose(Level.ERROR);
			else{
				System.err.println("\nERROR -> wrong level of verbosity (i:info (def), e:error, d:debug)\n");
				formatter.printHelp(new PrintWriter(System.err,true), 150, "mgclus -f INFILE [OPTIONS]\n", "", options, 2, 2, "");
				System.err.println("\n");
				System.exit(1);
			}
		}
	}
	

	private void setVerbose(Level verbose) {
		this.verbose = verbose;
	}

	public Level getVerbose() {
		return verbose;
	}

	private void setGraphFile(File graphFile) {
		this.graphFile = graphFile;
	}

	public File getGraphFile() {
		return graphFile;
	}

	private void setOutputFile(File outFile) {
		this.outFile = outFile;
	}

	public File getOutputFile() {
		return outFile;
	}

	private void setUseWeights(Boolean useWeights) {
		this.useWeights = useWeights;
	}

	public Boolean getUseWeights() {
		return useWeights;
	}

	private void setSeedsFile(File f) {
		this.seedsFile = f;
	}

	public File getSeedsFile() {
		return seedsFile;
	}
	
	private void setDelta(Double d) {
		this.delta = d;
	}

	public Double getDelta() {
		return delta;
	}
	
	
}
